;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 3 test for TEB bands. Calculates the OBC emissivity.
;
;  Input:
;  1) Input file with paths to RSR, RVS, and general info sav files as well
;     as the .temp.sav file.
;
;  Output:
;  1) emissivity[band,0=RSR 1=no RSR]
;  2) emissivity_2[collect,band,0=RSR 1=no RSR]
;  3) emissivity_3[collect,band,detector,subsample,0=RSR 1=no RSR]
;  4) emissivity_4[collect,band,HAM,detector,subsample,scan,0=RSR 1=no RSR]
;  5) rad_p1[band,collect,0=BCS 1=SVS,0=RSR 1=no RSR]
;  6) rad_BCS_scan[collect,band,HAM,detector,subsample,scan,0=RSR 1=no RSR]
;  7) rad_OBC_1_scan[collect,band,HAM,detector,subsample,scan,0=RSR 1=no RSR]
;  8) rad_OBC_scan[collect,band,HAM,detector,subsample,scan,0=RSR 1=no RSR]
;  9) rad_p2[band,HAM,collect,5,0=RSR 1=no RSR] -- 0 for HAM, 1 for RTA, 2 for CAV, 3 for SH, 
;     and 4 for OBC.
;  10) rad_p3[collects,band,HAM,detector,scans,5,0=RSR 1=no RSR] -- 0 for HAM, 1 for RTA, 2 for CAV, 3 for SH, 
;     and 4 for OBC.
;  11) rvs[band,subsample,detector,HAM,0=BCS 1=SVS]
;  12) ncollects
;  13) p -- band dependent reflectance.
;  
;  v1.0 - 04/01/2009 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_OBC_emiss, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
maxscan=intarr(1)               ; # of scans
sample1=intarr(1)               ; 1st M band sample
nsamples=intarr(1)              ; # of M band samples

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,maxscan
readf,lun,sample1
readf,lun,nsamples
Free_Lun,lun

max_scan=maxscan[0]

;*****************************************************************************
;  Read in temperatures, rvs, rsr, some general parameters and make an outfile.
;*****************************************************************************

restore,'general.TEB.2.sav'
restore,param_in
restore,rsrpath
restore,rvs_in

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.BCS.rg.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
outfile=outpath+'/'+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.sav'

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

Phi=dblarr(2)				; BCS and SVS.

rad_p1=dblarr(ncollects,nbands,maxdet,2,2)
rad_p2=dblarr(ncollects,nbands,nham,maxdet,5,2)
rad_p3=dblarr(ncollects,nbands,nham,maxdet,max_scan,5,2)

rvs=dblarr(nbands,maxsubsets,maxdet,nham,3)

emissivity_4=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,max_scan,2)
emissivity_3=dblarr(ncollects,nbands,maxdet,2)
emissivity_2=dblarr(ncollects,nbands,2)
emissivity=dblarr(nbands,2)

rad_BCS_scan=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,max_scan,2)
rad_OBC_1_scan=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,max_scan,2)
rad_OBC_scan=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,max_scan,2)

;*****************************************************************************
;  Calculate angles.
;*****************************************************************************

; Starting position of the collect window in samples.
win_num=4
MSample_offset=1064*win_num

Msample_min=sample1+MSample_offset
Msample_max=sample1+nsamples+MSample_offset

; Center sample numbers.
Msample_center=0.5*(Msample_max+Msample_min)

; Bore-sight scan angles; use M band (I and M band have same angles
Theta=((Msample_center-33.5)*0.017785-56.063)*(!pi/180)

Theta_SVS=-65.68*(!pi/180)
Theta_ref=46*(!pi/180)

; HAM angle of incidence.
Phi_perpendicular=28.6*(!pi/180)

Phi[0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta-Theta_ref)))*(180/!pi)
Phi[1]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_SVS-Theta_ref)))*(180/!pi)

;*****************************************************************************
;  Calculate the radiances, both integrating the RSR and L over wavelength 
;  and also the Planck radiance.
;*****************************************************************************

; Loop over BCS, rad_p1(i=0), SVS, rad_p1(i=1), HAM, rad_p3(i=0), RTA, rad_p2(i=1),
; CAV, rad_p2(i=2), SH, rad_p2(i=3), OBC, rad_p2(i=4). 

FOR col=0, ncollects-1 DO BEGIN
    IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

    FOR b=0, nbands-1 DO BEGIN
        FOR id=0, detectors[b]-1 DO BEGIN
            rad_p1[col,b,id,0,1]=Planck(T_bcs[col],lambda_center[b])
            rad_p1[col,b,id,1,1]=Planck(T_svs[col],lambda_center[b])
            FOR ih=0, nham-1 DO BEGIN
                rad_p2[col,b,ih,id,0,1]=Planck(T_ham[ih,col],lambda_center[b])
                rad_p2[col,b,ih,id,1,1]=Planck(T_rta[ih,col],lambda_center[b])
                rad_p2[col,b,ih,id,2,1]=Planck(T_cav[ih,col],lambda_center[b])
                rad_p2[col,b,ih,id,3,1]=Planck(T_sh[ih,col],lambda_center[b])
                rad_p2[col,b,ih,id,4,1]=Planck(T_obc[ih,col],lambda_center[b])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
FOR col=0, ncollects-1 DO BEGIN
    IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

    FOR b=0, nbands-1 DO BEGIN
        rad_p1[col,b,0:detectors[b]-1,0,0]=get_avg_2(rsrf,T_bcs[col],b,detectors[b])
        rad_p1[col,b,0:detectors[b]-1,1,0]=get_avg_2(rsrf,T_svs[col],b,detectors[b])
        FOR ih=0, nham-1 DO BEGIN
            rad_p2[col,b,ih,0:detectors[b]-1,0,0]=get_avg_2(rsrf,T_ham[ih,col],b,detectors[b])
            rad_p2[col,b,ih,0:detectors[b]-1,1,0]=get_avg_2(rsrf,T_rta[ih,col],b,detectors[b])
            rad_p2[col,b,ih,0:detectors[b]-1,2,0]=get_avg_2(rsrf,T_cav[ih,col],b,detectors[b])
            rad_p2[col,b,ih,0:detectors[b]-1,3,0]=get_avg_2(rsrf,T_sh[ih,col],b,detectors[b])
            rad_p2[col,b,ih,0:detectors[b]-1,4,0]=get_avg_2(rsrf,T_obc[ih,col],b,detectors[b])
        ENDFOR
    ENDFOR
ENDFOR

FOR iscan=0, max_scan-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR col=0, ncollects-1 DO BEGIN
                IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

                rad_p3[col,b,ih,0:detectors[b]-1,iscan,0,1]=Planck(T_ham_2[iscan,ih,col],lambda_center[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,1,1]=Planck(T_rta_2[iscan,ih,col],lambda_center[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,2,1]=Planck(T_cav_2[iscan,ih,col],lambda_center[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,3,1]=Planck(T_sh_2[iscan,ih,col],lambda_center[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,4,1]=Planck(T_obc_2[iscan,ih,col],lambda_center[b])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
FOR iscan=0, max_scan-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR col=0, ncollects-1 DO BEGIN
                IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

                rad_p3[col,b,ih,0:detectors[b]-1,iscan,0,0]=get_avg_2(rsrf,T_ham_2[iscan,ih,col],b,detectors[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,1,0]=get_avg_2(rsrf,T_rta_2[iscan,ih,col],b,detectors[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,2,0]=get_avg_2(rsrf,T_cav_2[iscan,ih,col],b,detectors[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,3,0]=get_avg_2(rsrf,T_sh_2[iscan,ih,col],b,detectors[b])
                rad_p3[col,b,ih,0:detectors[b]-1,iscan,4,0]=get_avg_2(rsrf,T_obc_2[iscan,ih,col],b,detectors[b])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the RVS.
;*****************************************************************************

; Restore RVS data called COEFF=dblarr(# bands,subsample,# detectors,HAM side,three fit parameters (ao,a1,a2)).
FOR b=0, nbands-1 DO BEGIN

    ; 0 for BCS and 1 for SVS.
    FOR j=0, 1 DO BEGIN
        IF band_type[b] EQ 1 THEN BEGIN
            rvs[b,*,*,*,j]=(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*Phi[j]+COEF[b+6,*,*,*,2]*Phi[j]^2) $
              /(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*38.53+COEF[b+6,*,*,*,2]*38.53^2)
        ENDIF
        IF band_type[b] EQ 0 THEN BEGIN
            rvs[b,*,*,*,j]=(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*Phi[j]+COEF[b-2,*,*,*,2]*Phi[j]^2) $
              /(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*38.53+COEF[b-2,*,*,*,2]*38.53^2)
        ENDIF
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the OBC emissivity.
;*****************************************************************************

;  0 for rsr correction and 1 for no rsr correction.
FOR rsr=0, 1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR col=0, ncollects-1 DO BEGIN
            IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

            FOR id=0, detectors[b]-1 DO BEGIN
                FOR is=0, subsets[b]-1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        IF dn_col[col,b,ham[iscan,col],id,0,1,iscan,0] GT dn_col[col,b,ham[iscan,col],id,1,1,iscan,0] $
                          AND dn_col[col,b,ham[iscan,col],id,0,1,iscan,2] GT dn_col[col,b,ham[iscan,col],id,1,1,iscan,2] THEN BEGIN 
                            isubset=is
                        ENDIF ELSE BEGIN
                            IF is EQ 0 THEN isubset=1
                            IF is EQ 1 THEN isubset=0
                        ENDELSE

                        temp1=(f_cav*rad_p3[col,b,ham[iscan,col],id,iscan,2,rsr]+f_sh*rad_p3[col,b,ham[iscan,col],id,iscan,3,rsr] $
                              +f_rta*rad_p3[col,b,ham[iscan,col],id,iscan,1,rsr])
                        temp2=(rg_1[b,ham[iscan,col],id,is,0,1,0]+rg_1[b,ham[iscan,col],id,is,1,1,0]*dn_col[col,b,ham[iscan,col],id,isubset,1,iscan,2] $
                               +rg_1[b,ham[iscan,col],id,is,2,1,0]*dn_col[col,b,ham[iscan,col],id,isubset,1,iscan,2]^2)
                        temp3=(rvs[b,is,id,ham[iscan,col],1]-1.)*(rad_p3[col,b,ham[iscan,col],id,iscan,0,rsr]-(1-p[b])*rad_p3[col,b,ham[iscan,col],id,iscan,1,rsr])

                        emissivity_4[col,b,ham[iscan,col],id,is,iscan,rsr] $
                          =(temp2+p[b]*rvs[b,is,id,ham[iscan,col],1]*rad_p1[col,b,id,1,rsr]-temp3-p[b]*temp1) $
                          /p[b]/(rad_p3[col,b,ham[iscan,col],id,iscan,4,rsr]-temp1)
                    ENDFOR
                ENDFOR
                emissivity_3[col,b,id,rsr]=sig_mean(emissivity_4[col,b,0:1,id,0:subsets[b]-1,0:max_scan-1,rsr],3,1)
            ENDFOR
            emissivity_2[col,b,rsr]=mean(emissivity_3[col,b,0:detectors[b]-1,rsr])
        ENDFOR
        xx=WHERE(emissivity_2[*,b,rsr] NE 0)
        emissivity[b,rsr]=mean(emissivity_2[xx,b,rsr])

        IF rsr EQ 0 THEN print,band_name[b],emissivity[b,rsr]
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the BCS and OBC radiances.
;*****************************************************************************

;  0 for rsr correction and 1 for no rsr correction.
FOR rsr=0, 1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR col=0, ncollects-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    rad_BCS_scan[col,b,ham[iscan,col],0:detectors[b]-1,is,iscan,rsr] $
                      =p[b]*(rvs[b,is,0:detectors[b]-1,ham[iscan,col],0]*rad_p1[col,b,0:detectors[b]-1,0,rsr] $
                      -rvs[b,is,0:detectors[b]-1,ham[iscan,col],1]*rad_p1[col,b,0:detectors[b]-1,1,rsr]) $
                      +(rvs[b,is,0:detectors[b]-1,ham[iscan,col],1]-rvs[b,is,0:detectors[b]-1,ham[iscan,col],0]) $
                      *(rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,0,rsr] $
                        -(1-p[b])*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,1,rsr])

                    rad_OBC_1_scan[col,b,ham[iscan,col],0:detectors[b]-1,is,iscan,rsr] $
                      =emiss_bb*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,4,rsr] $
                      +(1-emiss_bb)*(f_cav*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,2,rsr] $
                                     +f_sh*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,3,rsr] $
                                     +f_rta*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,1,rsr])

                    rad_OBC_scan[col,b,ham[iscan,col],0:detectors[b]-1,is,iscan,rsr] $
                      =p[b]*(rad_OBC_1_scan[col,b,ham[iscan,col],0:detectors[b]-1,is,iscan,rsr] $
                             -rvs[b,is,0:detectors[b]-1,ham[iscan,col],1]*rad_p1[col,b,0:detectors[b]-1,1,rsr]) $
                      +(rvs[b,is,0:detectors[b]-1,ham[iscan,col],1]-1.) $
                      *(rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,0,rsr]-(1-p[b])*rad_p3[col,b,ham[iscan,col],0:detectors[b]-1,iscan,1,rsr])
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

save,/compress,ncollects,emissivity,emissivity_2,emissivity_3,emissivity_4,rad_BCS_scan,rad_OBC_scan,rad_OBC_1_scan, $
  rad_p1,rad_p2,rad_p3,rvs,p,filename=outfile

;*****************************************************************************
;  Plot emissivities vs detector.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emissivity.ps'
device,/landscape,filename=outfname,/color
!p.multi=[0,1,1]
!p.position=0

;*****************************************************************************
;  Plot the emissivity per detector.
;*****************************************************************************

!p.multi=[0,1,1]
!p.position=0

G=findgen(40)
xtick=[' ',band_name[0:nbands-1],' ',' ']
title1='OBC emissivity (scan, HAM, and subsample averaged)'
FOR col=0, ncollects-1 DO BEGIN
    IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

    plot,G[0:nbands-1]+1,emissivity[0:nbands-1,0],xtit='Detector',ytit='Emissivity',yrange=[0.985,1.01], $
      xrange=[0,nbands+2],XTICKNAME=xtick,XTICKINTERVAL=1,charsize=1.2,charthick=5,thick=5,/nodata

    oplot,G[0:nbands-1]+1,emissivity[0:nbands-1,0],symsize=1,color=lcol[1],psym=symbol[1],thick=5
    oplot,G[0:nbands-1]+1,emissivity[0:nbands-1,0],color=lcol[1],thick=5

    oplot,[0.0,10.0],[1.0,1.0],linestyle=1,color=lcol[1],thick=5
    oplot,[0.0,10.0],[0.996,0.996],linestyle=1,color=lcol[1],thick=5

    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

title1='OBC emissivity (scan, HAM, and subsample averaged)'
FOR col=0, ncollects-1 DO BEGIN
    IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

    plot,G[0:detectors[0]-1]+1,emissivity_3[col,0,0:detectors[0]-1,0],xtit='Detector',ytit='Emissivity',yrange=[0.985,1.01], $
      xrange=[0,detectors[0]+2],charsize=1.2,charthick=5,thick=5,/nodata

    j=0
    FOR b=0, nbands-1 DO BEGIN
        oplot,G[0:detectors[b]-1]+1,emissivity_3[col,b,0:detectors[b]-1,0],symsize=1,color=lcol[j],psym=symbol[j],thick=5
        oplot,G[0:detectors[b]-1]+1,emissivity_3[col,b,0:detectors[b]-1,0],color=lcol[j],thick=5
        j=j+1
    ENDFOR
    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    set_legend,1,band_name,lcol,symbol
ENDFOR

FOR col=0, ncollects-1 DO BEGIN
    IF abs(T_obc[0,col]-T_bcs[col]) GT 1.0 THEN continue

    FOR b=0, nbands-1 DO BEGIN
        title2=band_name[b]+' collect='+strtrim(string(col+1),2)

        ymax=max(emissivity_3[col,b,0:detectors[b]-1,0],min=ymin)

        plot,G[0:detectors[b]-1]+1,emissivity_3[col,b,0:detectors[b]-1,0],xtit='Detector',ytit='Emissivity',yrange=[ymin,ymax], $
          xrange=[0,detectors[b]+2],title=title2,charsize=1.2,charthick=5,thick=5,/nodata

        oplot,G[0:detectors[b]-1]+1,emissivity_3[col,b,0:detectors[b]-1,0],symsize=1,color=lcol[1],psym=symbol[1],thick=5
        oplot,G[0:detectors[b]-1]+1,emissivity_3[col,b,0:detectors[b]-1,0],color=lcol[1],thick=5

        XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    ENDFOR
ENDFOR

device,/close

END


